-----------------------------------------------------------------------
-- Copyright (c) 2006, Impulse Accelerated Technologies, Inc.
-- All Rights Reserved.
--
-- apu440_if.vhd: Xilinx APU stream and UDI components
--
-- Change History
-- --------------
-- 7/8/2006 - Scott Thibault
--   File created.
-- 8/8/2006 - Scott Thibault
--   Introduced UDI adaptors
--
-----------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity apu_to_stream is
  generic (
    datawidth : positive := 8
  );
  port (
    reset, clk : in std_logic;
    apufcmce : std_logic;
    apufcminstruction : std_logic_vector(0 to 31);
    apufcminstrvalid : std_logic;
    apufcmradata : std_logic_vector(0 to 31);
    apufcmrbdata : std_logic_vector(0 to 31);
    apufcmoperandvalid : std_logic;
    apufcmflush : std_logic;
    apufcmwritebackok : std_logic;
    apufcmloaddata : std_logic_vector(0 to 127);
    apufcmloaddvalid : std_logic;
    apufcmloadbyteaddr : std_logic_vector(0 to 3);
    apufcmendian : std_logic;
    apufcmdecudi : std_logic_vector(0 to 3);
    apufcmdecudivalid : std_logic;
    fcmapustoredata : out std_logic_vector(0 to 127);
    fcmapudone : inout std_logic;       -- added "in"
    fcmapusleepnotready : out std_logic;
    fcmapuresultvalid : inout std_logic;-- added "in"
    stream_en : inout std_ulogic;       -- added "in"
    stream_rdy : in std_ulogic; 
    stream_eos : inout std_ulogic;        -- added "in"
    stream_data : out std_ulogic_vector (datawidth-1 downto 0)
  );
end apu_to_stream;

architecture rtl of apu_to_stream is
  type stateType is (STATE_IDLE, STATE_LOAD, STATE_LOAD_OK, STATE_STORE, STATE_STORE_OK);
  signal curr_state, next_state : stateType;
  signal stream_we : std_ulogic;
  signal error, write_addr, load_ok, store_ok : std_ulogic;
  signal status : std_ulogic_vector (31 downto 0);
  signal valid_instr, store_or_loadn: std_logic;
  signal target : std_logic_vector(4 downto 0);
  signal aligned : std_logic_vector(0 to 127);
begin

  target <= apufcminstruction(6 to 10);
    
  process (reset, clk)
  begin
    if (reset = '1') then
      curr_state <= STATE_IDLE;
    elsif (clk'event and clk='1') then
      curr_state <= next_state;
    end if;
  end process;

  store_or_loadn <= APUFCMINSTRUCTION(23);
  load_ok <=
    valid_instr and not store_or_loadn and apufcmloaddvalid and apufcmwritebackok when curr_state = STATE_IDLE else
    apufcmloaddvalid and apufcmwritebackok when curr_state = STATE_LOAD else
    apufcmloaddvalid when curr_state = STATE_LOAD_OK else
    '0';

-- Use the following for store with writeback
  store_ok <=
    apufcmwritebackok when curr_state = STATE_STORE else
    '1' when curr_state = STATE_STORE_OK else
    '0';

  valid_instr <= apufcminstrvalid and apufcmce and not apufcmflush;

  process (curr_state,valid_instr,apufcmflush,apufcmloaddvalid,apufcmwritebackok,store_or_loadn)
  begin
    case curr_state is
    when STATE_IDLE =>
      if (valid_instr = '1') then
        if (store_or_loadn = '1') then
          if (apufcmwritebackok = '1') then
            next_state <= STATE_STORE_OK;
          else
            next_state <= STATE_STORE;
          end if;
        else
          if (apufcmwritebackok = '1') then
            if (apufcmloaddvalid = '1') then
              next_state <= STATE_IDLE;
            else
              next_state <= STATE_LOAD_OK;
            end if;
          else
            next_state <= STATE_LOAD;
          end if;
        end if;
      else
        next_state <= STATE_IDLE;
      end if;
    when STATE_LOAD =>
      if (apufcmwritebackok = '1') then
        if (apufcmloaddvalid = '1') then
          next_state <= STATE_IDLE;
        else
          next_state <= STATE_LOAD_OK;
        end if;
      elsif (apufcmflush = '1') then
        next_state <= STATE_IDLE;
      else
        next_state <= STATE_LOAD;
      end if;
    when STATE_LOAD_OK =>
      if (apufcmloaddvalid = '1') then
        next_state <= STATE_IDLE;
      elsif (apufcmflush = '1') then
        next_state <= STATE_IDLE;
      else
        next_state <= STATE_LOAD_OK;
      end if;
    when STATE_STORE =>
      if apufcmwritebackok = '1' then
        next_state <= STATE_IDLE;
      elsif apufcmflush = '1' then
        next_state <= STATE_IDLE;
      else
        next_state <= STATE_STORE;
      end if;
    when STATE_STORE_OK =>
      next_state <= STATE_IDLE;
    end case;
  end process;

  steer8: if datawidth <= 8 generate
    aligned(0 to 7) <=
      apufcmloaddata(0 to 7) when apufcmloadbyteaddr = "0000" else
      apufcmloaddata(8 to 15) when apufcmloadbyteaddr = "0001" else
      apufcmloaddata(16 to 23) when apufcmloadbyteaddr = "0010" else
      apufcmloaddata(24 to 31) when apufcmloadbyteaddr = "0011" else
      apufcmloaddata(32 to 39) when apufcmloadbyteaddr = "0100" else
      apufcmloaddata(40 to 47) when apufcmloadbyteaddr = "0101" else
      apufcmloaddata(48 to 55) when apufcmloadbyteaddr = "0110" else
      apufcmloaddata(56 to 63) when apufcmloadbyteaddr = "0111" else
      apufcmloaddata(64 to 71) when apufcmloadbyteaddr = "1000" else
      apufcmloaddata(72 to 79) when apufcmloadbyteaddr = "1001" else
      apufcmloaddata(80 to 87) when apufcmloadbyteaddr = "1010" else
      apufcmloaddata(88 to 95) when apufcmloadbyteaddr = "1011" else
      apufcmloaddata(96 to 103) when apufcmloadbyteaddr = "1100" else
      apufcmloaddata(104 to 111) when apufcmloadbyteaddr = "1101" else
      apufcmloaddata(112 to 119) when apufcmloadbyteaddr = "1110" else
      apufcmloaddata(120 to 127);
  end generate;
  
  steer16: if datawidth > 8 and datawidth <= 16 generate
    aligned(0 to 15) <=
      apufcmloaddata(0 to 15) when apufcmloadbyteaddr = "0000" else
      apufcmloaddata(16 to 31) when apufcmloadbyteaddr = "0010" else
      apufcmloaddata(32 to 47) when apufcmloadbyteaddr = "0100" else
      apufcmloaddata(48 to 63) when apufcmloadbyteaddr = "0110" else
      apufcmloaddata(64 to 79) when apufcmloadbyteaddr = "1000" else
      apufcmloaddata(80 to 95) when apufcmloadbyteaddr = "1010" else
      apufcmloaddata(96 to 111) when apufcmloadbyteaddr = "1100" else
      apufcmloaddata(112 to 127);
  end generate;
  
  steer32: if datawidth > 16 and datawidth <= 32 generate
    aligned(0 to 31) <=
      apufcmloaddata(0 to 31) when apufcmloadbyteaddr = "0000" else
      apufcmloaddata(32 to 63) when apufcmloadbyteaddr = "0100" else
      apufcmloaddata(64 to 95) when apufcmloadbyteaddr = "1000" else
      apufcmloaddata(96 to 127);
  end generate;
  
  steer64: if datawidth > 32 and datawidth <= 64 generate
    aligned(0 to 63) <=
      apufcmloaddata(0 to 63) when apufcmloadbyteaddr = "0000" else
      apufcmloaddata(64 to 127);
  end generate;

  steer128: if datawidth > 64 generate
    aligned <= apufcmloaddata;
  end generate;
  
  write_addr <=
    '1' when target(1 downto 0) = "00" else
    '1' when target(1 downto 0) = "10" else
    '0';

  stream_we <= load_ok;

  -- Write to stream
  stream_en <= '1' when (stream_rdy = '1') and (stream_we = '1') and (write_addr = '1') else '0';
  stream_eos <= '1' when target(1 downto 0) = "10" else '0';
  stream_data <= std_ulogic_vector(aligned(0 to datawidth-1));

  -- Error detection
  check: process (reset, clk)
  begin
    if (reset = '1') then
      error <= '0';
    elsif (clk'event and clk='1') then
      if (stream_we = '1' and write_addr = '1') then
        error <= not stream_rdy;
      end if;
    end if;
  end process;
  
  -- Status register
  status(0) <= stream_rdy;
  status(1) <= '0';
  status(2) <= '0';
  status(3) <= error;
  status(31 downto 4) <= "0000000000000000000000000000";

  fcmapusleepnotready <= '1' when curr_state /= STATE_IDLE else '0';
  fcmapustoredata <=
    std_logic_vector(status & status & status & status) when (curr_state = STATE_STORE or curr_state = STATE_STORE_OK)
    else X"00000000000000000000000000000000";
  fcmapuresultvalid <=
    '1' when (curr_state = STATE_STORE or curr_state = STATE_STORE_OK) else
    '0';
  fcmapudone <= load_ok or store_ok;
end rtl;


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity stream_to_apu is
  generic (
    datawidth : positive := 8
  );
  port (
    reset, clk : in std_logic;
    apufcmce : std_logic;
    apufcminstruction : std_logic_vector(0 to 31);
    apufcminstrvalid : std_logic;
    apufcmradata : std_logic_vector(0 to 31);
    apufcmrbdata : std_logic_vector(0 to 31);
    apufcmoperandvalid : std_logic;
    apufcmflush : std_logic;
    apufcmwritebackok : std_logic;
    apufcmloaddata : std_logic_vector(0 to 127);
    apufcmloaddvalid : std_logic;
    apufcmloadbyteaddr : std_logic_vector(0 to 3);
    apufcmendian : std_logic;
    apufcmdecudi : std_logic_vector(0 to 3);
    apufcmdecudivalid : std_logic;
    fcmapustoredata : out std_logic_vector(0 to 127);
    fcmapudone : inout std_logic;
    fcmapusleepnotready : out std_logic;
    fcmapuresultvalid : inout std_logic;
    stream_en : inout std_ulogic;
    stream_rdy : in std_ulogic;
    stream_eos : in std_ulogic;
    stream_data : in std_ulogic_vector (datawidth-1 downto 0)
  );
end stream_to_apu;

architecture rtl of stream_to_apu is
  type stateType is (STATE_IDLE, STATE_STORE, STATE_WAIT_RDY, STATE_STORE_OK);
  signal curr_state, next_state : stateType;
  signal store_or_loadn, valid_instr, store_rdy, store_ok : std_logic;
  signal status : std_ulogic_vector (31 downto 0);
  signal extended_data : std_logic_vector (0 to 127);
  signal target : std_logic_vector(4 downto 0);
  signal aligned : std_logic_vector(0 to 127);
begin
  target <= apufcminstruction(6 to 10);

  process (reset, clk)
  begin
    if (reset = '1') then
      curr_state <= STATE_IDLE;
    elsif (clk'event and clk='1') then
      curr_state <= next_state;
    end if;
  end process;

  store_or_loadn <= apufcminstruction(23);

  valid_instr <= apufcminstrvalid and apufcmce and not apufcmflush;

  store_ok <=
    store_rdy and apufcmwritebackok when curr_state = STATE_STORE else
    store_rdy when curr_state = STATE_WAIT_RDY else
    '1' when curr_state = STATE_STORE_OK else
    '0';
  
  process (curr_state,valid_instr,store_or_loadn,store_rdy)
  begin
    case curr_state is
    when STATE_IDLE =>
      if (valid_instr = '1') then
        if (store_or_loadn = '1') then
          if (apufcmwritebackok = '1') then
            if (store_rdy = '1') then
              next_state <= STATE_STORE_OK;
            else
              next_state <= STATE_WAIT_RDY;
            end if;
          else
            next_state <= STATE_STORE;
          end if;
        end if;
      else
        next_state <= STATE_IDLE;
      end if;
    when STATE_STORE =>
      if apufcmwritebackok = '1' then
        if (store_rdy = '1') then
          next_state <= STATE_IDLE;
        else
          next_state <= STATE_WAIT_RDY;
        end if;
      elsif apufcmflush = '1' then
        next_state <= STATE_IDLE;
      else
        next_state <= STATE_STORE;
      end if;
    when STATE_WAIT_RDY =>
      if (store_rdy = '1') then
        next_state <= STATE_IDLE;
      else
        next_state <= STATE_WAIT_RDY;
      end if;
    when STATE_STORE_OK =>
      next_state <= STATE_IDLE;
    end case;
  end process;

  extended_data(0 to datawidth - 1) <= std_logic_vector(stream_data);

  steer8: if datawidth <= 8 generate
    aligned <=
      extended_data(0 to 7) &
      extended_data(0 to 7) &
      extended_data(0 to 7) &
      extended_data(0 to 7) &
      extended_data(0 to 7) &
      extended_data(0 to 7) &
      extended_data(0 to 7) &
      extended_data(0 to 7) &
      extended_data(0 to 7) &
      extended_data(0 to 7) &
      extended_data(0 to 7) &
      extended_data(0 to 7) &
      extended_data(0 to 7) &
      extended_data(0 to 7) &
      extended_data(0 to 7) &
      extended_data(0 to 7);
  end generate;

  steer16: if datawidth > 8 and datawidth <= 16 generate
    aligned <=
      extended_data(0 to 15) &
      extended_data(0 to 15) &
      extended_data(0 to 15) &
      extended_data(0 to 15) &
      extended_data(0 to 15) &
      extended_data(0 to 15) &
      extended_data(0 to 15) &
      extended_data(0 to 15);
  end generate;

  steer32: if datawidth > 16 and datawidth <= 32 generate
    aligned <=
      extended_data(0 to 31) &
      extended_data(0 to 31) &
      extended_data(0 to 31) &
      extended_data(0 to 31);
  end generate;

  steer64: if datawidth > 32 and datawidth <= 64 generate
    aligned <=
      extended_data(0 to 63) &
      extended_data(0 to 63);
  end generate;

  steer128: if datawidth > 64 generate
    aligned <= extended_data;
  end generate;

  -- Status register
  status(0) <= stream_rdy;
  status(1) <= stream_eos;
  status(2) <= '0';
  status(3) <= '0';
  status(31 downto 4) <= "0000000000000000000000000000";

  stream_en <= store_ok when target(1 downto 0) = "00" else '0';

  store_rdy <= stream_rdy when target(1 downto 0) = "00" else '1';
  
  fcmapusleepnotready <= '1' when curr_state /= STATE_IDLE else '0';
  fcmapustoredata <=
    aligned when curr_state /= STATE_IDLE and target(1 downto 0) = "00" else
    std_logic_vector(status & status & status & status) when curr_state /= STATE_IDLE else
    X"00000000000000000000000000000000";
  fcmapuresultvalid <= store_rdy when curr_state /= STATE_IDLE else '0';
  fcmapudone <= store_ok;
end rtl;

library ieee;
use ieee.std_logic_1164.all;

entity autonomous_udi_ctrl is
  generic (
    UDI_NUM : std_logic_vector(0 to 3) := "0000";
    NUM_OPERANDS : natural := 0
    );
  port (
    reset, clk : in std_logic;
    apufcmdecudivalid : in std_logic;
    apufcmdecudi : in std_logic_vector(0 to 3);
    apufcmoperandvalid : in std_logic;
    apufcmflush : in std_logic;
    apufcmwritebackok : in std_logic;
    impcdone : in std_ulogic;
    fcmapudone : out std_logic;
    fcmapusleepnotready : out std_logic;
    impcrun : out std_ulogic
    );
end autonomous_udi_ctrl;

architecture rtl of autonomous_udi_ctrl is
  type stateType is (idle, wait_input, wait_ok, wait_complete);
  signal state, nextState : stateType;
  signal newinstr : std_ulogic;
  signal data_ready : std_ulogic;
begin
  process (reset, clk)
  begin
    if (reset = '1') then
      state <= idle;
    else
      if (clk = '1' and clk'event) then
        state <= nextState;
      end if;
    end if;
  end process;

  newinstr <= '1' when apufcmdecudivalid = '1' and apufcmdecudi = UDI_NUM else '0';

  without_operands: if NUM_OPERANDS = 0 generate
    data_ready <= '1';
  end generate without_operands;
  with_operands: if NUM_OPERANDS > 0 generate
    data_ready <= not apufcmoperandvalid;
  end generate with_operands;
  
  process (state,newinstr,data_ready,apufcmwritebackok,apufcmflush,impcdone)
  begin
    case state is
    when idle =>
      if newinstr = '1' and apufcmflush = '0' then
        if apufcmwritebackok = '1' then
          if data_ready = '1' then
            nextState <= wait_complete;
          else
            nextState <= wait_input;
          end if;
        else
          nextState <= wait_ok;
        end if;
      else
        nextState <= idle;
      end if;
    when wait_ok =>
      -- have not received writebackok or operandvalid
      if apufcmwritebackok = '1' then
        if data_ready = '1' then
          nextState <= wait_complete;
        else
          nextState <= wait_input;
        end if;
      elsif apufcmflush = '1' then
        nextState <= idle;
      else
        nextState <= wait_ok;
      end if;
    when wait_input =>
      -- have not received operandvalid
      if data_ready = '1' then
        nextState <= wait_complete;
      else
        nextState <= wait_input;
      end if;
    when wait_complete =>
      if impcdone = '1' then
        nextState <= idle;
      else
        nextState <= wait_complete;
      end if;
    end case;
  end process;
  
  impcrun <=
    '1' when nextState = wait_complete or state = wait_complete else
    '0';
  
  fcmapudone <= impcdone when state = wait_complete else '0';
  fcmapusleepnotready <= '0' when state = idle else '1';
end rtl;

library ieee;
use ieee.std_logic_1164.all;

entity blocking_udi_ctrl is
  generic (
    UDI_NUM : std_logic_vector(0 to 3) := "0000";
    NUM_OPERANDS : natural := 0
    );
  port (
    reset, clk : in std_logic;
    apufcmdecudivalid : in std_logic;
    apufcmdecudi : in std_logic_vector(0 to 3);
    apufcmoperandvalid : in std_logic;
    apufcmflush : in std_logic;
    apufcmwritebackok : in std_logic;
    impcdone : in std_ulogic;
    fcmapuresultvalid : out std_logic;
    fcmapudone : out std_logic;
    fcmapusleepnotready : out std_logic;
    impcrun : out std_ulogic
    );
end blocking_udi_ctrl;

architecture rtl of blocking_udi_ctrl is
  type stateType is (idle, wait_input, wait_input_ok, confirm, wait_complete, send_result);
  signal state, nextState : stateType;
  signal newinstr : std_ulogic;
  signal need_operands : std_ulogic;
begin
  process (reset, clk)
  begin
    if (reset = '1') then
      state <= idle;
    else
      if (clk = '1' and clk'event) then
        state <= nextState;
      end if;
    end if;
  end process;

  newinstr <= '1' when apufcmdecudivalid = '1' and apufcmdecudi = UDI_NUM else '0';

  need_operands <= '0' when NUM_OPERANDS = 0 else '1';

  process (state,newinstr,need_operands,apufcmoperandvalid,apufcmwritebackok,apufcmflush,impcdone)
  begin
    case state is
    when idle =>
      if newinstr = '1' then
        if need_operands = '1' then
          if apufcmwritebackok = '1' then
            nextState <= wait_input_ok;
          else
            nextState <= wait_input;
          end if;
        else
          if apufcmwritebackok = '1' then
            nextState <= wait_complete;
          else
            nextState <= confirm;
          end if;
        end if;
      else
        nextState <= idle;
      end if;
    when wait_input =>
      -- have not received writebackok or operandvalid
      if apufcmwritebackok = '1' then
        if apufcmoperandvalid = '1' then
          nextState <= wait_complete;
        else
          nextState <= wait_input_ok;
        end if;
      elsif apufcmflush = '1' then
        nextState <= idle;
      else
        if apufcmoperandvalid = '1' then
          nextState <= confirm;
        else
          nextState <= wait_input;
        end if;
      end if;
    when wait_input_ok =>
      -- have not received operandvalid
      if apufcmoperandvalid = '1' then
        nextState <= wait_complete;
      else
        nextState <= wait_input_ok;
      end if;
    when confirm =>
      -- have not received writebackok
      if apufcmwritebackok = '1' then
        nextState <= wait_complete;
      elsif apufcmflush = '1' then
        nextState <= idle;
      else
        nextState <= confirm;
      end if;
    when wait_complete =>
      if impcdone = '1' then
        nextState <= send_result;
      else
        nextState <= wait_complete;
      end if;
    when send_result =>
      -- point results arrive in the cycle after the acknowledge signal
      nextState <= idle;
    end case;
  end process;

  impcrun <=
    '1' when nextState = wait_complete or state = wait_complete else
    '0';
  
  fcmapuresultvalid <= '1' when state = send_result else '0';
  fcmapudone <= '1' when state = send_result else '0';
  fcmapusleepnotready <= '0' when state = idle else '1';
end rtl;

library ieee;
use ieee.std_logic_1164.all;

entity nonblocking_udi_ctrl is
  generic (
    UDI_NUM : std_logic_vector(0 to 3) := "0000";
    NUM_OPERANDS : natural := 0
    );
  port (
    reset, clk : in std_logic;
    apufcmdecudivalid : in std_logic;
    apufcmdecudi : in std_logic_vector(0 to 3);
    apufcmoperandvalid : in std_logic;
    apufcmflush : in std_logic;
    apufcmwritebackok : in std_logic;
    impcdone : in std_ulogic;
    fcmapuresultvalid : out std_logic;
    fcmapudone : out std_logic;
    fcmapusleepnotready : out std_logic;
    impcrun : out std_ulogic
    );
end nonblocking_udi_ctrl;

architecture rtl of nonblocking_udi_ctrl is
  type stateType is (idle, wait_input, wait_input_ok, confirm, wait_complete, send_result);
  signal state, nextState : stateType;
  signal newinstr : std_ulogic;
  signal need_operands : std_ulogic;
begin
  process (reset, clk)
  begin
    if (reset = '1') then
      state <= idle;
    else
      if (clk = '1' and clk'event) then
        state <= nextState;
      end if;
    end if;
  end process;

  newinstr <= '1' when apufcmdecudivalid = '1' and apufcmdecudi = UDI_NUM else '0';

  need_operands <= '0' when NUM_OPERANDS = 0 else '1';

  process (state,newinstr,need_operands,apufcmoperandvalid,apufcmwritebackok,apufcmflush,impcdone)
  begin
    case state is
    when idle =>
      if newinstr = '1' then
        if need_operands = '1' then
          if apufcmwritebackok = '1' then
            nextState <= wait_input_ok;
          else
            nextState <= wait_input;
          end if;
        else
          if apufcmwritebackok = '1' then
            nextState <= wait_complete;
          else
            nextState <= confirm;
          end if;
        end if;
      else
        nextState <= idle;
      end if;
    when wait_input =>
      -- have not received writebackok or operandvalid
      if apufcmwritebackok = '1' then
        if apufcmoperandvalid = '1' then
          nextState <= wait_complete;
        else
          nextState <= wait_input_ok;
        end if;
      elsif apufcmflush = '1' then
        nextState <= idle;
      else
        if apufcmoperandvalid = '1' then
          nextState <= confirm;
        else
          nextState <= wait_input;
        end if;
      end if;
    when wait_input_ok =>
      -- have not received operandvalid
      if apufcmoperandvalid = '1' then
        nextState <= wait_complete;
      else
        nextState <= wait_input_ok;
      end if;
    when confirm =>
      -- have not received writebackok
      if apufcmwritebackok = '1' then
        nextState <= wait_complete;
      elsif apufcmflush = '1' then
        nextState <= idle;
      else
        nextState <= confirm;
      end if;
    when wait_complete =>
      if impcdone = '1' then
        nextState <= send_result;
      else
        nextState <= wait_complete;
      end if;
    when send_result =>
      -- point results arrive in the cycle after the acknowledge signal
      nextState <= idle;
    end case;
  end process;

  impcrun <=
    '1' when nextState = wait_complete or state = wait_complete else
    '0';
  
  fcmapuresultvalid <= '1' when state = send_result else '0';
  fcmapudone <= '1' when state = send_result else '0';
  fcmapusleepnotready <= '0' when state = idle else '1';
end rtl;

library ieee;
use ieee.std_logic_1164.all;

package apu_if is

  component apu_to_stream is
  generic (
    datawidth : positive := 8
  );
  port (
    reset, clk : in std_logic;
    apufcmce : std_logic;
    apufcminstruction : std_logic_vector(0 to 31);
    apufcminstrvalid : std_logic;
    apufcmradata : std_logic_vector(0 to 31);
    apufcmrbdata : std_logic_vector(0 to 31);
    apufcmoperandvalid : std_logic;
    apufcmflush : std_logic;
    apufcmwritebackok : std_logic;
    apufcmloaddata : std_logic_vector(0 to 127);
    apufcmloaddvalid : std_logic;
    apufcmloadbyteaddr : std_logic_vector(0 to 3);
    apufcmendian : std_logic;
    apufcmdecudi : std_logic_vector(0 to 3);
    apufcmdecudivalid : std_logic;
    fcmapustoredata : out std_logic_vector(0 to 127);
    fcmapudone : inout std_logic;
    fcmapusleepnotready : out std_logic;
    fcmapuresultvalid : inout std_logic;
    stream_en : inout std_ulogic;
    stream_rdy : in std_ulogic;
    stream_eos : inout std_ulogic;
    stream_data : out std_ulogic_vector (datawidth-1 downto 0)
  );
  end component;

  component stream_to_apu is
  generic (
    datawidth : positive := 8
  );
  port (
    reset, clk : in std_logic;
    apufcmce : std_logic;
    apufcminstruction : std_logic_vector(0 to 31);
    apufcminstrvalid : std_logic;
    apufcmradata : std_logic_vector(0 to 31);
    apufcmrbdata : std_logic_vector(0 to 31);
    apufcmoperandvalid : std_logic;
    apufcmflush : std_logic;
    apufcmwritebackok : std_logic;
    apufcmloaddata : std_logic_vector(0 to 127);
    apufcmloaddvalid : std_logic;
    apufcmloadbyteaddr : std_logic_vector(0 to 3);
    apufcmendian : std_logic;
    apufcmdecudi : std_logic_vector(0 to 3);
    apufcmdecudivalid : std_logic;
    fcmapustoredata : out std_logic_vector(0 to 127);
    fcmapudone : inout std_logic;
    fcmapusleepnotready : out std_logic;
    fcmapuresultvalid : inout std_logic;
    stream_en : inout std_ulogic;
    stream_rdy : in std_ulogic;
    stream_eos : in std_ulogic;
    stream_data : in std_ulogic_vector (datawidth-1 downto 0)
  );
  end component;

  component autonomous_udi_ctrl is
  generic (
    UDI_NUM : std_logic_vector(0 to 3);
    NUM_OPERANDS : natural := 0
    );
  port (
    reset, clk : in std_logic;
    apufcmdecudivalid : in std_logic;
    apufcmdecudi : in std_logic_vector(0 to 3);
    apufcmoperandvalid : in std_logic;
    apufcmflush : in std_logic;
    apufcmwritebackok : in std_logic;
    impcdone : in std_ulogic;
    fcmapudone : out std_logic;
    fcmapusleepnotready : out std_logic;
    impcrun : out std_ulogic
    );
  end component;

  component blocking_udi_ctrl is
  generic (
    UDI_NUM : std_logic_vector(0 to 3);
    NUM_OPERANDS : natural := 0
    );
  port (
    reset, clk : in std_logic;
    apufcmdecudivalid : in std_logic;
    apufcmdecudi : in std_logic_vector(0 to 3);
    apufcmoperandvalid : in std_logic;
    apufcmflush : in std_logic;
    apufcmwritebackok : in std_logic;
    impcdone : in std_ulogic;
    fcmapuresultvalid : out std_logic;
    fcmapudone : out std_logic;
    fcmapusleepnotready : out std_logic;
    impcrun : out std_ulogic
    );
  end component;

  component nonblocking_udi_ctrl is
  generic (
    UDI_NUM : std_logic_vector(0 to 3);
    NUM_OPERANDS : natural := 0
    );
  port (
    reset, clk : in std_logic;
    apufcmdecudivalid : in std_logic;
    apufcmdecudi : in std_logic_vector(0 to 3);
    apufcmoperandvalid : in std_logic;
    apufcmflush : in std_logic;
    apufcmwritebackok : in std_logic;
    impcdone : in std_ulogic;
    fcmapuresultvalid : out std_logic;
    fcmapudone : out std_logic;
    fcmapusleepnotready : out std_logic;
    impcrun : out std_ulogic
    );
  end component;

end package;
